==> s-loc-instance-0.1.7.tgz: package/lib/instance-hooks.js <==
let Manager = require('./ec2-instance-manager');
let AWS = require('aws-sdk'), Dynamo = new AWS.DynamoDB();
let uuid = require('node-uuid');

function FetchItemParams(from) {
    return {
        "instance_id": {
            "S" : from.instance_id
        }
    }
}

function UpdateChars(from) {
    let update = {};
    for (let i = 0; i < from.params.length; i++) {
        let tag = from.params[i];
        let value = from[tag];
        switch (typeof value ){
            case 'string' :
                update[tag] = {
                    "S" : value
                };
                break;
            case 'number' :
                update[tag] = {
                    "N" : value
                };
                break;
            default:
                break;
        }
    }
    return update;
}

function UpdateAsSet(from) {
    let updateables = ["merchant_name"];
    let UpdateExpression = "";
    let ExpressionAttributeValues = {};
    let ExpressionAttributeNames = {};
    let update = {
        TableName: "loc_tables_"+from.region
    };
    let keys = UpdateChars(from);
    for (let i = 0; i < updateables.length; i++) {
        let tag = updateables[i];
        let value = from[tag];
        let exptag = "#"+i;
        let valtag = ":"+i;
        switch (typeof value ){
            case 'string' :
                ExpressionAttributeValues[valtag] = {
                    "S" : value
                };
                break;
            case 'number' :
                ExpressionAttributeValues[valtag] = {
                    "N" : value
                };
                break;
            default:
                break;
        }
        ExpressionAttributeNames[exptag] = tag;
        if (i < updateables.length - 1) {
            UpdateExpression += " "+exptag+" = "+valtag+",";
        } else {
            UpdateExpression += " "+exptag+" = "+valtag;
        }
    }
    update.Key = keys;
    update.ExpressionAttributeValues = ExpressionAttributeValues;
    update.ExpressionAttributeNames = ExpressionAttributeNames;
    update.UpdateExpression = "SET"+UpdateExpression;
    console.log(update);
    return update;
}

let inst = require('./aws/instance');

function InstanceHookError(error) {
    this.error= error;
    this._at = new Date();
    this._id = uuid.v4();
}


class InstanceHook extends Manager {
    constructor(params){
        super(params);
        this.errorType = InstanceHookError;
        this.reapplying_configs = false;
        if (params && params.attribute_blocks && typeof params.attribute_blocks === 'object') {
            this['attribute_blocks'] = params.attribute_blocks
        }
    }
    parseHook() {
        let it = this;
        return new Promise(async function (result) {
            try {
                await it.parse();
                await HandleNewParamsAndGetBucket();
                it.dynamo = new AWS.DynamoDB({apiVersion: "2012-08-10", region: it.region});
                let apply = it.attribute_blocks.fetch_main;
                if (!apply) throw "Missing Fetch Attributes For Parse Hook - main";
                let found = await it.fetchMain();
                if (!found) {
                    let values = {};
                    await ConstructValuesForNoRecordDispatcher();
                    it.Instance = new inst(values);
                    function ConstructValuesForNoRecordDispatcher() {
                        for (let i = 0; i < apply.length; i++){
                            let tag = apply[i];
                            values[tag] = it[tag]
                        }
                    }
                } else it.Instance = new inst(found);
                await ParseCurrentParams();
                await it.Instance.update();
                return result(Promise.resolve());
                function ParseCurrentParams() {
                    for (let i = 0; i < it.params.length; i++) {
                        let parameter = it.params[i], value = it[parameter];
                        if (parameter && value) {
                            it.Instance.set(parameter, value);
                        }
                    }
                    delete it.params;
                    RemoveIterimParams();
                }
            } catch(e) {
                return result(Promise.reject(new InstanceHookError(e)));
            }
        });
        function RemoveIterimParams() {
            let params = Object.keys(it.Instance.params);
            for (let i = 0; i < params.length; i++) {
                delete it[params[i]];
            }

        }
        function HandleNewParamsAndGetBucket() {
            return new Promise(function (handled) {
                if (!it.region) return handled(Promise.reject(new InstanceHookError('No Region Found in Config')));
                it.bucket = new AWS.S3({apiVersion: '2006-03-01', region: it.region});
                it.bucket.getObject({
                    Bucket : it.region+'-dispatcher',
                    Key: 'config/dispatch_config_'+it.region+'.json'
                }, function (err, data) {
                    if (err) return handled(Promise.reject(new InstanceHookError(err)));
                    if (!data.Body) return handled(Promise.reject(new InstanceHookError('ERROR: No Body Present In Body')));
                    if (!data.Body) return handled(Promise.reject(new InstanceHookError('ERROR: No Body Buffer Data Present In Body')));
                    let bod = data.Body.toString(), JSONResult;
                    if (!bod) return handled(Promise.reject(new InstanceHookError('ERROR: Failed to Parse Data')));
                    JSONResult = JSON.parse(bod);
                    if (!JSONResult.tablename) return handled(Promise.reject(new InstanceHookError('ERROR: No TableName Found in Params')));
                    it.tablename = JSONResult.tablename;
                    if (JSONResult.attribute_blocks && typeof JSONResult.attribute_blocks === 'object') {
                        it['attribute_blocks'] = JSONResult.attribute_blocks
                    } else return handled(Promise.reject(new InstanceHookError('No Attribute Block Found In Params')));
                    if (JSONResult.instances_tables) {
                        it['instances_tables'] = JSONResult.instances_tables
                    } else return handled(Promise.reject(new InstanceHookError('No Attribute Block Found In Params')));
                    return handled(Promise.resolve());
                });
            })
        }
    }

    reapplyConfig(region, it) {
        return new Promise(function (handled) {
            if (!region) return handled(Promise.reject(new InstanceHookError('No Region Found in Instance')));
            if (!it || !it.bucket) return handled(Promise.reject(new InstanceHookError('No Bucketeer Found in Instance')));
            it.reapplying_configs = true;
            it.bucket.getObject({
                Bucket : it.region+'-dispatcher',
                Key: '/config/dispatch_config_'+it.region+'.json'
            }, function (err, data) {
                if (err) return handled(Promise.reject(new InstanceHookError(err)));
                if (!data.Body) return handled(Promise.reject(new InstanceHookError('ERROR: No Body Present In Body')));
                let bod = data.Body.toString(), JSONResult;
                if (!bod) return handled(Promise.reject(new InstanceHookError('ERROR: Failed to Parse Data')));
                JSONResult = JSON.parse(bod);
                if (!JSONResult.tablename) return handled(Promise.reject(new InstanceHookError('ERROR: No TableName Found in Params')));
                it.tablename = JSONResult.tablename;
                if (JSONResult.attribute_blocks && typeof JSONResult.attribute_blocks === 'object') {
                    it['attribute_blocks'] = JSONResult.attribute_blocks
                } else return handled(Promise.reject(new InstanceHookError('No Attribute Block Found In Params')));
                if (JSONResult.instances_tables) {
                    it['instances_tables'] = JSONResult.instances_tables
                } else return handled(Promise.reject(new InstanceHookError('No Attribute Block Found In Params')));
                return handled(Promise.resolve());
            });
        })
    }

    fetchBucketObject(key, toType) {
        let it = this;
        return new Promise(function (result) {
            if (!key) return result(Promise.reject(new InstanceHookError('No Key Supplied')));
            if (!it.Instance.region) return result(Promise.reject(new InstanceHookError('No Region Found in Instance')));
            if (!it.bucket) return result(Promise.reject(new InstanceHookError('No Bucketeer Found in Instance')));
            it.bucket.getObject({
                Bucket: it.Instance.region+'-dispatcher',
                Key: key
            }, async function (err, data) {
                console.log('S3 Error', key);
                if (err) return result(Promise.reject(new InstanceHookError(err)));
                if (!data.Body) return result(Promise.reject(new InstanceHookError('ERROR: No Body Present In Body')));
                let bod = data.Body.toString(), JSONResult;
                if (!bod) return result(Promise.reject(new InstanceHookError('ERROR: Failed to Parse Data')));
                try {
                    if (!toType) JSONResult = await JSON.parse(bod);
                    return result(Promise.resolve(JSONResult));
                } catch(e) {
                    return result(Promise.reject(new InstanceHookError(e)));
                }
            })
        })
    }


    fetchParams(name) {
        let it = (!this.Instance) ? this : this.Instance;
        let attributes = (!name) ? this.attribute_blocks.fetch_main : this.attribute_blocks[name];
        let key = FetchItemParams(this);
        let Table = (!it.tablename) ? ((!it.Instance) ? null : it.Instance.source.tablename) : it.tablename;
        if (!attributes) return "Missing Key Attributes For "+((!name) ? "Main" : name);
        if (!key) return "Missing Key For Fetch in Params";
        if (!Table) return "Missing TableName Attribute In Params";
        return {
            Key: key,
            AttributesToGet: attributes,
            TableName: Table+it.region
        }
    }

    fetchOnInstance(type, on) {
        let it = this;
        let perform = (!on) ? it.Instance : on;
        let fetch = it.fetchParams()//it.attribute_blocks.fetch_main;
        console.log(fetch, 'fetching is fetch');
        if (!fetch) throw "Missing Fetch Attributes";
        if (typeof fetch === 'string') return new Promise(function (result) {
            return result(Promise.reject('Invalid Fetch Params '+fetch));
        });
        if (!type) return new Promise(function (result) {
            perform.dynamo.getItem(fetch, function (err, got) {
                return result((err) ? Promise.reject(new InstanceHookError(err)) : ((!got.Item) ? null : got.Item));
            })
        });
        switch (type) {
            case 'get' : return new Promise(function (result) {
                perform.dynamo.getItem(fetch, function (err, got) {
                    return result((err) ? Promise.reject(new InstanceHookError(err)) : ((!got.Item) ? null : got.Item));
                })
            });
            case 'scan' :
                return new Promise(function (result) {
                    perform.dynamo.scan({}, function (err, got) {
                        return result((err) ? Promise.reject(new InstanceHookError(err)) : ((!got.Items) ? null : got.Item));
                    })
                });
            default: return;
        }
    }

    updateInstanceParametersFromInitial() {
        let it = this;
        return new Promise(function (result) {
            CheckParams(it).then(result).catch(result);
        });
        async function CheckParams(it) {
            try {
                let params_on_root = it.root_instance_params;
                if (!params_on_root) throw "Error : Root Instance Params Not Configured Before Update";
                let currents = params_on_root.parameters;
                if (!currents) {
                    throw "Error - No Parameters Set For Instance Root Item"
                } else {
                    await ParseCurrentParams([
                        'ami_id',
                        'local_hostname',
                        'local_ipv4',
                        'public_hostname',
                        'public_ipv4',
                        'security_groups',
                        'region'
                    ]);
                }
                async function ParseCurrentParams(keys) {
                    let err;
                    for (let i = 0; i < keys.length; i++) {
                        let key = keys[i], value = (it.Instance) ? it.Instance[key] : it[key];
                        if (!value) {
                            err = "Error - Parameter "+key+" Not Defined";break;
                        }
                        params_on_root.updateParameter(key, value)
                    }
                    if (err) throw err;
                    return Promise.resolve();
                }

            } catch(e) {
                throw e;
            }

        }
    }

    updateRootDBWithInstanceParams() {
        let it = this;
        return new Promise(async function (result) {
            try {
                await it.updateInstanceParametersFromInitial();
                let params = await it.root_instance_params.toDynamoUpdateObject();
                it.root_dynamo.updateItem(params, function (err, done) {
                    if (err) return result(new it.errorType(err));
                    return result(Promise.resolve())
                })
            } catch(e) {
                return result(new it.errorType(e));
            }
        })
    }
        

    

    async fetchMain() {
        try {
            return await this.fetchOnInstance('get', this);
        } catch(e) {
            throw e;
        }
    }
}

module.exports = InstanceHook;

==> s-loc-instance-5.0.39.tgz: package/lib/instance-hooks.js <==
let Generator = require('aws-generator'),
    Manager = Generator.InstanceDataManager,
    AWS = Generator.AWS,
    inst = Generator.Instance,
    uuid = Generator.NodeUUID;

function FetchItemParams(from) {
    return {
        "instance_id": {
            "S" : from.instance_id
        }
    }
}

function UpdateChars(from) {
    let update = {};
    for (let i = 0; i < from.params.length; i++) {
        let tag = from.params[i];
        let value = from[tag];
        switch (typeof value ){
            case 'string' :
                update[tag] = {
                    "S" : value
                };
                break;
            case 'number' :
                update[tag] = {
                    "N" : value
                };
                break;
            default:
                break;
        }
    }
    return update;
}

function UpdateAsSet(from) {
    let updateables = ["merchant_name"];
    let UpdateExpression = "";
    let ExpressionAttributeValues = {};
    let ExpressionAttributeNames = {};
    let update = {
        TableName: "loc_tables_"+from.region
    };
    let keys = UpdateChars(from);
    for (let i = 0; i < updateables.length; i++) {
        let tag = updateables[i];
        let value = from[tag];
        let exptag = "#"+i;
        let valtag = ":"+i;
        switch (typeof value ){
            case 'string' :
                ExpressionAttributeValues[valtag] = {
                    "S" : value
                };
                break;
            case 'number' :
                ExpressionAttributeValues[valtag] = {
                    "N" : value
                };
                break;
            default:
                break;
        }
        ExpressionAttributeNames[exptag] = tag;
        if (i < updateables.length - 1) {
            UpdateExpression += " "+exptag+" = "+valtag+",";
        } else {
            UpdateExpression += " "+exptag+" = "+valtag;
        }
    }
    update.Key = keys;
    update.ExpressionAttributeValues = ExpressionAttributeValues;
    update.ExpressionAttributeNames = ExpressionAttributeNames;
    update.UpdateExpression = "SET"+UpdateExpression;
    console.log(update);
    return update;
}


function InstanceHookError(error) {
    this.error= error;
    this._at = new Date();
    this._id = uuid.v4();
}


class InstanceHook extends Manager {
    constructor(params){
        super(params);
        this.errorType = InstanceHookError;
        this.reapplying_configs = false;
        if (params && params.attribute_blocks && typeof params.attribute_blocks === 'object') {
            this['attribute_blocks'] = params.attribute_blocks
        }
    }
    parseHook() {
        let it = this;
        return new Promise(async function (result) {
            try {
                await it.parse();
                await HandleNewParamsAndGetBucket();
                it.dynamo = Generator.Generators//new AWS.DynamoDB({apiVersion: "2012-08-10", region: it.region});
                let apply = it.attribute_blocks.fetch_main;
                if (!apply) throw "Missing Fetch Attributes For Parse Hook - main";
                let found = await it.fetchMain();
                if (!found) {
                    let values = {};
                    await ConstructValuesForNoRecordDispatcher();
                    it.Instance = new inst(values);
                    function ConstructValuesForNoRecordDispatcher() {
                        for (let i = 0; i < apply.length; i++){
                            let tag = apply[i];
                            values[tag] = it[tag]
                        }
                    }
                } else it.Instance = new inst(found);
                await ParseCurrentParams();
                await it.Instance.update();
                return result(Promise.resolve());
                function ParseCurrentParams() {
                    for (let i = 0; i < it.params.length; i++) {
                        let parameter = it.params[i], value = it[parameter];
                        if (parameter && value) {
                            it.Instance.set(parameter, value);
                        }
                    }
                    delete it.params;
                    RemoveIterimParams();
                }
            } catch(e) {
                return result(Promise.reject(new InstanceHookError(e)));
            }
        });
        function RemoveIterimParams() {
            let params = Object.keys(it.Instance.params);
            for (let i = 0; i < params.length; i++) {
                delete it[params[i]];
            }

        }
        function HandleNewParamsAndGetBucket() {
            return new Promise(function (handled) {
                if (!it.region) return handled(Promise.reject(new InstanceHookError('No Region Found in Config')));
                it.bucket = new AWS.S3({apiVersion: '2006-03-01', region: it.region});
                it.bucket.getObject({
                    Bucket : it.region+'-dispatcher',
                    Key: 'config/dispatch_config_'+it.region+'.json'
                }, function (err, data) {
                    if (err) return handled(Promise.reject(new InstanceHookError(err)));
                    if (!data.Body) return handled(Promise.reject(new InstanceHookError('ERROR: No Body Present In Body')));
                    if (!data.Body) return handled(Promise.reject(new InstanceHookError('ERROR: No Body Buffer Data Present In Body')));
                    let bod = data.Body.toString(), JSONResult;
                    if (!bod) return handled(Promise.reject(new InstanceHookError('ERROR: Failed to Parse Data')));
                    JSONResult = JSON.parse(bod);
                    if (!JSONResult.tablename) return handled(Promise.reject(new InstanceHookError('ERROR: No TableName Found in Params')));
                    it.tablename = JSONResult.tablename;
                    if (JSONResult.attribute_blocks && typeof JSONResult.attribute_blocks === 'object') {
                        it['attribute_blocks'] = JSONResult.attribute_blocks
                    } else return handled(Promise.reject(new InstanceHookError('No Attribute Block Found In Params')));
                    if (JSONResult.instances_tables) {
                        it['instances_tables'] = JSONResult.instances_tables
                    } else return handled(Promise.reject(new InstanceHookError('No Attribute Block Found In Params')));
                    return handled(Promise.resolve());
                });
            })
        }
    }

    reapplyConfig(region, it) {
        return new Promise(function (handled) {
            if (!region) return handled(Promise.reject(new InstanceHookError('No Region Found in Instance')));
            if (!it || !it.bucket) return handled(Promise.reject(new InstanceHookError('No Bucketeer Found in Instance')));
            it.reapplying_configs = true;
            it.bucket.getObject({
                Bucket : it.region+'-dispatcher',
                Key: '/config/dispatch_config_'+it.region+'.json'
            }, function (err, data) {
                if (err) return handled(Promise.reject(new InstanceHookError(err)));
                if (!data.Body) return handled(Promise.reject(new InstanceHookError('ERROR: No Body Present In Body')));
                let bod = data.Body.toString(), JSONResult;
                if (!bod) return handled(Promise.reject(new InstanceHookError('ERROR: Failed to Parse Data')));
                JSONResult = JSON.parse(bod);
                if (!JSONResult.tablename) return handled(Promise.reject(new InstanceHookError('ERROR: No TableName Found in Params')));
                it.tablename = JSONResult.tablename;
                if (JSONResult.attribute_blocks && typeof JSONResult.attribute_blocks === 'object') {
                    it['attribute_blocks'] = JSONResult.attribute_blocks
                } else return handled(Promise.reject(new InstanceHookError('No Attribute Block Found In Params')));
                if (JSONResult.instances_tables) {
                    it['instances_tables'] = JSONResult.instances_tables
                } else return handled(Promise.reject(new InstanceHookError('No Attribute Block Found In Params')));
                return handled(Promise.resolve());
            });
        })
    }

    fetchBucketObject(key, toType) {
        let it = this;
        return new Promise(function (result) {
            if (!key) return result(Promise.reject(new InstanceHookError('No Key Supplied')));
            if (!it.Instance.region) return result(Promise.reject(new InstanceHookError('No Region Found in Instance')));
            if (!it.bucket) return result(Promise.reject(new InstanceHookError('No Bucketeer Found in Instance')));
            it.bucket.getObject({
                Bucket: it.Instance.region+'-dispatcher',
                Key: key
            }, async function (err, data) {
                console.log('S3 Error', key);
                if (err) return result(Promise.reject(new InstanceHookError(err)));
                if (!data.Body) return result(Promise.reject(new InstanceHookError('ERROR: No Body Present In Body')));
                let bod = data.Body.toString(), JSONResult;
                if (!bod) return result(Promise.reject(new InstanceHookError('ERROR: Failed to Parse Data')));
                try {
                    if (!toType) JSONResult = await JSON.parse(bod);
                    return result(Promise.resolve(JSONResult));
                } catch(e) {
                    return result(Promise.reject(new InstanceHookError(e)));
                }
            })
        })
    }


    fetchParams(name) {
        let it = (!this.Instance) ? this : this.Instance;
        let attributes = (!name) ? this.attribute_blocks.fetch_main : this.attribute_blocks[name];
        let key = FetchItemParams(this);
        let Table = (!it.tablename) ? ((!it.Instance) ? null : it.Instance.source.tablename) : it.tablename;
        if (!attributes) return "Missing Key Attributes For "+((!name) ? "Main" : name);
        if (!key) return "Missing Key For Fetch in Params";
        if (!Table) return "Missing TableName Attribute In Params";
        return {
            Key: key,
            AttributesToGet: attributes,
            TableName: Table+it.region
        }
    }

    fetchOnInstance(type, on) {
        let it = this;
        let perform = (!on) ? it.Instance : on;
        let fetch = it.fetchParams()//it.attribute_blocks.fetch_main;
        console.log(fetch, 'fetching is fetch');
        if (!fetch) throw "Missing Fetch Attributes";
        if (typeof fetch === 'string') return new Promise(function (result) {
            return result(Promise.reject('Invalid Fetch Params '+fetch));
        });
        if (!type) return new Promise(function (result) {
            perform.dynamo.getItem(fetch, function (err, got) {
                return result((err) ? Promise.reject(new InstanceHookError(err)) : ((!got.Item) ? null : got.Item));
            })
        });
        switch (type) {
            case 'get' : return new Promise(function (result) {
                perform.dynamo.getItem(fetch, function (err, got) {
                    return result((err) ? Promise.reject(new InstanceHookError(err)) : ((!got.Item) ? null : got.Item));
                })
            });
            case 'scan' :
                return new Promise(function (result) {
                    perform.dynamo.scan({}, function (err, got) {
                        return result((err) ? Promise.reject(new InstanceHookError(err)) : ((!got.Items) ? null : got.Item));
                    })
                });
            default: return;
        }
    }

    updateInstanceParametersFromInitial() {
        let it = this;
        return new Promise(function (result) {
            CheckParams(it).then(result).catch(result);
        });
        async function CheckParams(it) {
            try {
                let params_on_root = it.root_instance_params;
                if (!params_on_root) throw "Error : Root Instance Params Not Configured Before Update";
                let currents = params_on_root.parameters;
                if (!currents) {
                    throw "Error - No Parameters Set For Instance Root Item"
                } else {
                    await ParseCurrentParams([
                        'ami_id',
                        'local_hostname',
                        'local_ipv4',
                        'public_hostname',
                        'public_ipv4',
                        'security_groups',
                        'region'
                    ]);
                }
                async function ParseCurrentParams(keys) {
                    let err;
                    for (let i = 0; i < keys.length; i++) {
                        let key = keys[i], value = (it.Instance) ? it.Instance[key] : it[key];
                        if (!value) {
                            err = "Error - Parameter "+key+" Not Defined";break;
                        }
                        params_on_root.updateParameter(key, value)
                    }
                    if (err) throw err;
                    return Promise.resolve();
                }

            } catch(e) {
                throw e;
            }

        }
    }

    updateRootDBWithInstanceParams() {
        let it = this;
        return new Promise(async function (result) {
            try {
                await it.updateInstanceParametersFromInitial();
                let params = await it.root_instance_params.toDynamoUpdateObject();
                it.root_dynamo.updateItem(params, function (err, done) {
                    if (err) return result(new it.errorType(err));
                    return result(Promise.resolve())
                })
            } catch(e) {
                return result(new it.errorType(e));
            }
        })
    }
        

    

    async fetchMain() {
        try {
            return await this.fetchOnInstance('get', this);
        } catch(e) {
            throw e;
        }
    }
}

module.exports = InstanceHook;
